@@ -449,6 +449,7 @@ def model_remove_Identity(m, f: str):
         G = Graph(onnxg)
         G.save_model(f)
 
+
 def model_shape_infer(m, dynamic_shapes: {str: tuple} = None,
                       saveshapesmodel: str = None, shapesonly: bool = False, verbose: bool = False,
                       dump_outputs: [str] = None):
@@ -526,7 +527,7 @@ def print_node_map(f: str = None, metric='MACs'):
         row.append('{:.2%}'.format(item['macs'] / macs))
         row.append(num2str(int(item['memory']), csvformat))
         row.append('{:.2%}'.format(item['memory'] / memory))
-        row.append(num2str(int(item['params'])))
+        row.append(num2str(int(item['params']), csvformat))
         row.append('{:.2%}'.format(item['params'] / params))
         row.append(tuple2str(item['inshape'], splitch))
         row.append(tuple2str(item['outshape'], splitch))
@@ -725,6 +726,15 @@ def model_simplify_names(m, savemodel: str, renametensor=True, renamelayer=True,
         G.save_model(savemodel)
 
 
+def model_reorder_nodes(m, savemodel: str, ):
+    if isinstance(m, str):
+        m = onnx.load_model(m)
+    if isinstance(m, onnx.ModelProto):
+        G = Graph(m.graph)
+        G = G.graph_reorder()
+        G.save_model(savemodel)
+
+
 def model_io_modify(m, savemodel: str, custom_io):
     '''
         Args:
