@@ -51,6 +51,14 @@ def conv_output_shape(xin, pad, ksize, stride, dilation):
     return int((xin + pad - dilation * (ksize - 1) - 1) / stride + 1)
 
 
+def auto_pad_valid_shape_calc(x, ksize, stride):
+    return math.ceil((x - ksize + 1) / stride)
+
+
+def auto_pad_same_shape_calc(x, stride):
+    return math.ceil((x) / stride)
+
+
 def convtranspose_output_shape(xin, output_padding, pad, ksize, stride, dilation):
     return stride * (xin - 1) + output_padding + ((ksize - 1) * dilation + 1) - pad
 
@@ -94,10 +102,17 @@ class Conv(NodeBase):
         wshape = wtensor.shape
         shape = []
         if self.auto_pad is not None and self.auto_pad != b'NOTSET':
+            assert (self.dilations[0] == 1 and self.dilations[1] == 1)  # dilation=1 only
             if self.auto_pad in [b'SAME_LOWER', b'SAME_UPPER']:
                 shape = (xshape[0], wshape[0], math.ceil(xshape[2] / self.strides[0]))
                 if len(xshape) == 4:
                     shape += (math.ceil(xshape[3] / self.strides[1]),)
+            elif self.auto_pad == b'VALID':
+                oh = math.ceil((xshape[2] - wshape[2] + 1) / self.strides[0])
+                shape = (xshape[0], wshape[0], oh)
+                if len(xshape) == 4:
+                    ow = math.ceil((xshape[3] - wshape[3] + 1) / self.strides[1])
+                    shape += (ow,)
         else:
             if len(xshape) == 4:
                 oh = conv_output_shape(xshape[2], self.pads[0] + self.pads[2], wshape[2], self.strides[0],
@@ -324,10 +339,14 @@ def scatter_nd_impl(data, indices, updates, reduction='none'):  # type: ignore
     output = numpy.copy(data)
     for i in numpy.ndindex(indices.shape[:-1]):
         # NOTE: The order of iteration in this loop is not specified.
-        if reduction == 'add':
+        if reduction == "add":
             output[indices[i]] += updates[i]
-        elif reduction == 'mul':
+        elif reduction == "mul":
             output[indices[i]] *= updates[i]
+        elif reduction == "max":
+            output[indices[i]] = np.maximum(output[indices[i]], updates[i])
+        elif reduction == "min":
+            output[indices[i]] = np.minimum(output[indices[i]], updates[i])
         else:
             output[indices[i]] = updates[i]
     return output
@@ -460,7 +479,8 @@ def auto_add_attributes(atts, attnames, obj):
 class PoolBase(NodeBase):
     def __init__(self, nodeproto):
         super().__init__(nodeproto)
-        validAttr = ['kernel_shape', 'pads', 'strides', 'ceil_mode']
+        validAttr = ['kernel_shape', 'pads', 'strides', 'ceil_mode', 'auto_pad']
+        self.auto_pad = None
         self.kernel_shape = (3, 3)
         self.ceil_mode = 0
         self.pads = (0, 0, 0, 0)
@@ -470,23 +490,35 @@ class PoolBase(NodeBase):
         self.op_mac = CMP_MACS
 
     def infer_shape(self, intensors: [numpy.ndarray]):
-        if len(self.kernel_shape) == 1:
+        if self.auto_pad is not None and self.auto_pad != b'NOTSET':
             inshape = intensors[0].shape
-            outshape = inshape[:2] + (
-                pooling_shape_calc(inshape[2], self.pads[0] + self.pads[1], self.kernel_shape[0], self.dilations[0],
-                                   self.strides[0], self.ceil_mode),)
-            return [create_ndarray_f32(outshape)]
-        if len(self.kernel_shape) == 2:
-            inshape = intensors[0].shape
-            outshape = inshape[:2] + (
-                pooling_shape_calc(inshape[2], self.pads[0] + self.pads[2], self.kernel_shape[0], self.dilations[0],
-                                   self.strides[0],
-                                   self.ceil_mode),
-                pooling_shape_calc(inshape[3], self.pads[1] + self.pads[3], self.kernel_shape[1], self.dilations[1],
-                                   self.strides[1],
-                                   self.ceil_mode),
-            )
-            return [create_ndarray_f32(outshape)]
+            outshape = inshape[:2]
+            if self.auto_pad in [b'SAME_LOWER', b'SAME_UPPER']:
+                outshape += (auto_pad_same_shape_calc(inshape[2], self.strides[0]),)
+                if len(self.strides) == 2:
+                    outshape += (auto_pad_same_shape_calc(inshape[3], self.strides[1]),)
+            elif self.auto_pad == b'VALID':
+                outshape += (auto_pad_valid_shape_calc(inshape[2], self.kernel_shape[0], self.strides[0]),)
+                if len(self.strides) == 2:
+                    outshape += (auto_pad_valid_shape_calc(inshape[3], self.kernel_shape[1], self.strides[1]),)
+        else:
+            if len(self.kernel_shape) == 1:
+                inshape = intensors[0].shape
+                outshape = inshape[:2] + (
+                    pooling_shape_calc(inshape[2], self.pads[0] + self.pads[1], self.kernel_shape[0], self.dilations[0],
+                                       self.strides[0], self.ceil_mode),)
+                return [create_ndarray_f32(outshape)]
+            if len(self.kernel_shape) == 2:
+                inshape = intensors[0].shape
+                outshape = inshape[:2] + (
+                    pooling_shape_calc(inshape[2], self.pads[0] + self.pads[2], self.kernel_shape[0], self.dilations[0],
+                                       self.strides[0],
+                                       self.ceil_mode),
+                    pooling_shape_calc(inshape[3], self.pads[1] + self.pads[3], self.kernel_shape[1], self.dilations[1],
+                                       self.strides[1],
+                                       self.ceil_mode),
+                )
+        return [create_ndarray_f32(outshape)]
 
     def profile(self, intensors: [numpy.ndarray], outtensors: [numpy.ndarray]):
         outvol = volume(outtensors[0].shape)
@@ -778,7 +810,7 @@ def scatter_elements(data, indices, updates, axis=0, reduction='none'):  # type:
     if axis < 0:
         axis = data.ndim + axis
 
-    idx_xsection_shape = indices.shape[:axis] + indices.shape[axis + 1:]
+    idx_xsection_shape = indices.shape[:axis] + indices.shape[axis + 1 :]
 
     def make_slice(arr, axis, i):  # type: ignore
         slc = [slice(None)] * arr.ndim
@@ -792,36 +824,52 @@ def scatter_elements(data, indices, updates, axis=0, reduction='none'):  # type:
         return unpacked
 
     def make_indices_for_duplicate(idx):  # type: ignore
-        final_idx = ()
+        final_idx = list()
         for i in range(len(idx[0])):
             final_idx.append(tuple(idx_element[i] for idx_element in idx))
-        return (final_idx)
+        return list(final_idx)
 
     # We use indices and axis parameters to create idx
     # idx is in a form that can be used as a NumPy advanced indices for scattering of updates param. in data
-    idx = [[unpack(np.indices(idx_xsection_shape).reshape(indices.ndim - 1, -1)),
-            indices[tuple(make_slice(indices, axis, i))].reshape(1, -1)[0]] for i in range(indices.shape[axis])]
-    idx = (np.concatenate(idx, axis=1))
+    idx = [
+        [
+            unpack(np.indices(idx_xsection_shape).reshape(indices.ndim - 1, -1)),
+            indices[tuple(make_slice(indices, axis, i))].reshape(1, -1)[0],
+        ]
+        for i in range(indices.shape[axis])
+    ]
+    idx = list(np.concatenate(idx, axis=1))
     idx.insert(axis, idx.pop())
 
     # updates_idx is a NumPy advanced indices for indexing of elements in the updates
-    updates_idx = (idx)
+    updates_idx = list(idx)
     updates_idx.pop(axis)
-    updates_idx.insert(axis, np.repeat(np.arange(indices.shape[axis]), np.prod(idx_xsection_shape)))
+    updates_idx.insert(
+        axis, np.repeat(np.arange(indices.shape[axis]), np.prod(idx_xsection_shape))
+    )
 
     scattered = np.copy(data)
-    if reduction == 'none':
+    if reduction == "none":
         scattered[tuple(idx)] = updates[tuple(updates_idx)]
     else:
-        idx, updates_idx = make_indices_for_duplicate(idx), make_indices_for_duplicate(updates_idx)
+        idx, updates_idx = make_indices_for_duplicate(idx), make_indices_for_duplicate(
+            updates_idx
+        )
         for iter, idx_set in enumerate(idx):
-            if reduction == 'add':
+            if reduction == "add":
                 scattered[idx_set] += updates[updates_idx[iter]]
-            elif reduction == 'mul':
+            elif reduction == "mul":
                 scattered[idx_set] *= updates[updates_idx[iter]]
+            elif reduction == "max":
+                scattered[idx_set] = np.maximum(
+                    scattered[idx_set], updates[updates_idx[iter]]
+                )
+            elif reduction == "min":
+                scattered[idx_set] = np.minimum(
+                    scattered[idx_set], updates[updates_idx[iter]]
+                )
     return scattered
 
-
 @NODEPROFILER_REGISTRY.register()
 class ScatterElements(NodeBase):
     def __init__(self, node):
@@ -1402,6 +1450,13 @@ class Mul(PWNBase):
         return [intensors[0] * intensors[1]]
 
 
+@NODEPROFILER_REGISTRY.register()
+class ImageScaler(PWNBase):
+    def __init__(self, node_proto):
+        super().__init__(node_proto)
+        self.op_mac = ADD_MACS + MUL_MACS
+        self.ratio = 1
+
 @NODEPROFILER_REGISTRY.register()
 class InstanceNormalization(PWNBase):
     def __init__(self, node_proto):
@@ -1637,6 +1692,18 @@ class QLinearConv(Conv):
         return macs, params
 
 
+@NODEPROFILER_REGISTRY.register()
+class ArrayFeatureExtractor(NodeBase):
+    def infer_shape(self, intensors: [numpy.ndarray]):
+        return [intensors[1]]
+
+
+@NODEPROFILER_REGISTRY.register()
+class ZipMap(NodeBase):
+    def infer_shape(self, intensors: [numpy.ndarray]):
+        return [create_ndarray_f32((intensors[0].shape[0]), )]
+
+
 def node_profile(node_proto: str, ins: [], outs: []):
     node_class = NODEPROFILER_REGISTRY.get(node_proto.op_type)
     if node_class != None:
